#!/bin/bash
DESC="initialization of working counter for Quality Assurance"

PATH=/sbin:/bin:/usr/sbin:/usr/bin

PROPCLI="/usr/sbin/propcli"
SYNCH_FILE="/config/last_synch"
UPTIME_FILE="/config/uptime"
TAG="WorkingCounter"

if [ ! -x ${PROPCLI} ]; then
    echo "${PROPCLI} binary is missing!"
    exit 1
fi

wait_ipc()
{
    echo -n "Waiting for property service... "
    while [ true ]; do
	${PROPCLI} get System.QA.Startups &>/dev/zero
	EXITCODE="$?"
        if [ ${EXITCODE} -eq 0 ]; then
	    echo
	    break
	fi
	usleep 200000
	echo -n "."
	COUNTER=$((COUNTER+1))
	if [ ${COUNTER} -gt $1 ]; then
	    echo -e "\nCannot communicate with property service!"
	    exit 1
	fi
    done
}

do_onStart()
{
    wait_ipc 10
    TMP=`${PROPCLI} get System.QA.Startups`
    TMP=$((TMP+1))
    ${PROPCLI} set System.QA.Startups $TMP

	if [ ! -f $SYNCH_FILE ]; then
		echo 0 > $SYNCH_FILE
	fi

	COUNTER_IN_HOURS=`${PROPCLI} get System.QA.WorkingCounter`
	STORED_UPTIME=`${PROPCLI} get System.QA.WorkingCounterSec`
	LAST_SYNCH=$(cat $SYNCH_FILE)

	# Update working counter in seconds from old counter in hours if needed
	# Working counter in seconds will be 0 only after first sw update
	if [ $STORED_UPTIME -eq 0 ]; then
		STORED_UPTIME=$((COUNTER_IN_HOURS*3600))
		logger -t $TAG "Set working timer to $STORED_UPTIME sec"
		${PROPCLI} set System.QA.WorkingCounterSec $STORED_UPTIME
	fi

	if [ -f $UPTIME_FILE ]; then
		UPTIME=$(cat $UPTIME_FILE)
		TMP=$((UPTIME+STORED_UPTIME-LAST_SYNCH))
		logger -t $TAG "Uptime from file: $UPTIME, stored working counter: $STORED_UPTIME, last synch: $LAST_SYNCH"
		logger -t $TAG "New value of working counter: $TMP"

		${PROPCLI} set System.QA.WorkingCounterSec $TMP
		rm -f $UPTIME_FILE
	fi

	echo 0 > $SYNCH_FILE
    sync
}

do_onSoftReset()
{
    wait_ipc 2
    TMP=`${PROPCLI} get System.QA.SoftReboots`
    TMP=$((TMP+1))
    ${PROPCLI} set System.QA.SoftReboots $TMP
    sync
}

case "$1" in
 start)
	do_onStart
	;;
 stop)
	do_onSoftReset
	;;
 *)
	echo "Usage: $0 [ start | stop ] " >&2
	exit 1
	;;
esac

exit 0
