#!/bin/bash
DESC="A parsing script for an extraction and storing the last ntp synchronization timestamp in a volatile memory"

LOG_FILE="/var/volatile/ntpsync"
NTP_STATUS_CMD='ntpq -p'
NTP_TIME_ELAPSED_COLUMN='$5'
NTPQ_SERVER_DATA_LINE=3
IS_INTEGER_REGEX='^[0-9]+$'
TIME_SPAN=120

write_last_ntp_sync_to_file()
{
	if [[ $ntp_last_sync_delay =~ $IS_INTEGER_REGEX ]]; then
		current_timestamp=`date +%s`
		last_sync_timestamp="$(( $current_timestamp - $ntp_last_sync_delay ))"
		echo "$last_sync_timestamp" > ${LOG_FILE}
	else
		echo "Error couldn't get a valid value for ntp last sync. The current value is \"$ntp_last_sync_delay\"" >&2
	fi
}

repeat_periodically()
{
	while [ true ]
	do
		sleep $TIME_SPAN
		ntp_last_sync_delay=`${NTP_STATUS_CMD} | awk "{ if (NR == ${NTPQ_SERVER_DATA_LINE}) print ${NTP_TIME_ELAPSED_COLUMN}};"`
		write_last_ntp_sync_to_file
	done
}

repeat_periodically
